/* access macros for Q40 ISA bridge */

#ifndef _Q40_IO_H_
#define _Q40_IO_H_

#if 1
#define q40_isa_io_base  0xff400000
#define q40_isa_mem_base 0xff800000
#else
extern unsigned long q40_isa_io_base;
extern unsigned long q40_isa_mem_base;
#endif

#define ISA_IO_B(_pcadr_)  (q40_isa_io_base+1+4*((unsigned)_pcadr_))
#define ISA_IO_W(_pcadr_)  (q40_isa_io_base+4*((unsigned)_pcadr_))

#define ISA_MEM_B(_pcadr_)  (q40_isa_mem_base+1+4*((unsigned)_pcadr_))
#define ISA_MEM_W(_pcadr_)  (q40_isa_mem_base+4*((unsigned)_pcadr_))

/* IO space */

#define isa_inb(addr) \
    ({ unsigned char __v = (*(volatile unsigned char *) ISA_IO_B(addr)); __v; })

#define isa_outb(b,addr) (void)((*(volatile unsigned char *) ISA_IO_B(addr)) = (b))

#if 0  /* unused for now ... */
#define isa_inw(addr) \
    ({ unsigned short __v = (*(volatile unsigned short *) ISA_IO_W(addr)); __v; })
#define isa_outw(b,addr) (void)((*(volatile unsigned short *) ISA_IO_W(addr)) = (b))
#endif


/* mem space */

#define readb(addr) \
    ({ unsigned char __v = (*(volatile unsigned char *) ISA_MEM_B(addr)); __v; })
#define readw(addr) \
    ({ unsigned short __v = (*(volatile unsigned short *) ISA_MEM_W(addr)); __v; })


#define writeb(b,addr) (void)((*(volatile unsigned char *) ISA_MEM_B(addr)) = (b))
#define writew(b,addr) (void)((*(volatile unsigned short *) ISA_MEM_W(addr)) = (b))



/* now for the general case */

#define inb  isa_inb
#define outb isa_outb

#if 1
#define inb_p  isa_inb
#define outb_p isa_outb
#else
extern void q40_slow_io(void);  /* see if that is enough */
#define SLOW_DOWN_IO   do{q40_slow_io();}while(0)
static inline unsigned char _inb_p(unsigned long p)
{
  unsigned char c;

  SLOW_DOWN_IO;
  c=isa_inb(p);
  SLOW_DOWN_IO;
  return c;
}
static inline void _outb_p(unsigned char c, unsigned long addr)
{
  SLOW_DOWN_IO;
  isa_outb(c,addr);
  SLOW_DOWN_IO;
}
#define inb_p(__addr__)           _inb_p((unsigned long)(__addr__))
#define outb_p(__val__,__addr__)  _outb_p(__val__,(unsigned long)(__addr__))
#endif

#endif /* _Q40_IO_H_ */
